@@ -3,10 +3,12 @@
 import gtk
 import gettext
 
-from tryton.config import TRYTON_ICON
-from tryton.common import get_toplevel_window
-from tryton.common.datetime_strftime import datetime_strftime
+from tryton.common import get_toplevel_window, IconFactory
 from tryton.common.datetime_ import date_parse
+from tryton.common.datetime_strftime import datetime_strftime
+from tryton.common.underline import set_underline
+from tryton.config import TRYTON_ICON
+from tryton.gui import Main
 
 _ = gettext.gettext
 
@@ -17,9 +19,18 @@ class Revision(object):
     def __init__(self, revisions, revision=None, format_='%x %H:%M:%S.%f'):
         self.parent = get_toplevel_window()
         self.win = gtk.Dialog(_('Revision'), self.parent,
-            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                gtk.STOCK_OK, gtk.RESPONSE_OK))
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+        Main().add_window(self.win)
+        cancel_button = self.win.add_button(
+            set_underline(_("Cancel")), gtk.RESPONSE_CANCEL)
+        cancel_button.set_image(IconFactory.get_image(
+                'tryton-cancel', gtk.ICON_SIZE_BUTTON))
+        cancel_button.set_always_show_image(True)
+        ok_button = self.win.add_button(
+            set_underline(_("OK")), gtk.RESPONSE_OK)
+        ok_button.set_image(IconFactory.get_image(
+                'tryton-ok', gtk.ICON_SIZE_BUTTON))
+        ok_button.set_always_show_image(True)
         self.win.set_default_response(gtk.RESPONSE_OK)
         self.win.set_icon(TRYTON_ICON)
         self.win.vbox.set_spacing(3)
