@@ -8,11 +8,11 @@ import datetime
 import calendar
 import json
 import collections
-import urllib
-import urlparse
+import urllib.parse
 import xml.dom.minidom
 import gettext
 import logging
+from operator import itemgetter
 
 import gtk
 
@@ -27,7 +27,6 @@ from tryton.common import RPCExecute, RPCException, MODELACCESS, \
     node_attributes, sur, RPCContextReload, warning
 from tryton.action import Action
 from tryton.pyson import PYSONDecoder
-import tryton.rpc as rpc
 
 _ = gettext.gettext
 logger = logging.getLogger(__name__)
@@ -129,8 +128,6 @@ class Screen(SignalEvent):
         self.search_value = attributes.get('search_value')
         self.fields_view_tree = {}
         self.order = self.default_order = attributes.get('order')
-        self.__date_format = self.context.get(
-            'date_format', rpc.CONTEXT.get('locale', {}).get('date', '%x'))
         self.view_to_load = []
         self._domain_parser = {}
         self.pre_validate = False
@@ -172,7 +169,7 @@ class Screen(SignalEvent):
             view_tree = self.fields_view_tree[view_id]
 
         fields = copy.deepcopy(view_tree['fields'])
-        for name, props in fields.iteritems():
+        for name, props in fields.items():
             if props['type'] not in ('selection', 'reference'):
                 continue
             if isinstance(props['selection'], (tuple, list)):
@@ -183,11 +180,26 @@ class Screen(SignalEvent):
             # Filter only fields in XML view
             xml_dom = xml.dom.minidom.parseString(view_tree['arch'])
             root_node, = xml_dom.childNodes
-            xml_fields = [node_attributes(node).get('name')
-                for node in root_node.childNodes
-                if node.nodeName == 'field']
-            fields = collections.OrderedDict(
-                (name, fields[name]) for name in xml_fields)
+            ofields = collections.OrderedDict()
+            for node in root_node.childNodes:
+                if node.nodeName != 'field':
+                    continue
+                attributes = node_attributes(node)
+                name = attributes['name']
+                # If a field is defined multiple times in the XML,
+                # take only the first definition
+                if name in ofields:
+                    continue
+                ofields[name] = fields[name]
+                for attr in ['string', 'factor']:
+                    if attributes.get(attr):
+                        ofields[name][attr] = attributes[attr]
+            fields = ofields
+
+        if 'active' in view_tree['fields']:
+            self.screen_container.but_active.show()
+        else:
+            self.screen_container.but_active.hide()
 
         # Add common fields
         for name, string, type_ in (
@@ -199,16 +211,14 @@ class Screen(SignalEvent):
                 ):
             if name not in fields:
                 fields[name] = {
-                    'string': string.decode('utf-8'),
+                    'string': string,
                     'name': name,
                     'type': type_,
                     }
                 if type_ == 'datetime':
                     fields[name]['format'] = '"%H:%M:%S"'
 
-        context = rpc.CONTEXT.copy()
-        context.update(self.context)
-        domain_parser = DomainParser(fields, context)
+        domain_parser = DomainParser(fields, self.context)
         self._domain_parser[view_id] = domain_parser
         return domain_parser
 
@@ -223,7 +233,7 @@ class Screen(SignalEvent):
                     props['selection'])
         except RPCException:
             selection = []
-        selection.sort(lambda x, y: cmp(x[1], y[1]))
+        selection.sort(key=itemgetter(1))
         return selection
 
     def search_prev(self, search_string):
@@ -258,16 +268,19 @@ class Screen(SignalEvent):
         if tab_domain:
             domain = ['AND', domain, tab_domain]
 
+        context = self.context
+        if self.screen_container.but_active.get_active():
+            context['active_test'] = False
         try:
             ids = RPCExecute('model', self.model_name, 'search', domain,
-                self.offset, self.limit, self.order, context=self.context)
+                self.offset, self.limit, self.order, context=context)
         except RPCException:
             ids = []
         if not only_ids:
             if self.limit is not None and len(ids) == self.limit:
                 try:
                     self.search_count = RPCExecute('model', self.model_name,
-                        'search_count', domain, context=self.context)
+                        'search_count', domain, context=context)
                 except RPCException:
                     self.search_count = 0
             else:
@@ -306,6 +319,11 @@ class Screen(SignalEvent):
         else:
             domain = self.domain
 
+        if self.screen_container.but_active.get_active():
+            if domain:
+                domain = [domain, ('active', '=', False)]
+            else:
+                domain = [('active', '=', False)]
         if self.current_view and self.current_view.view_type == 'calendar':
             if domain:
                 domain = ['AND', domain, self.current_view.current_domain()]
@@ -333,21 +351,22 @@ class Screen(SignalEvent):
 
     @property
     def context(self):
-        return self.group.context
-
-    @property
-    def date_format(self):
-        return self.__date_format
+        context = self.group.context
+        if self.context_screen:
+            context['context_model'] = self.context_screen.model_name
+        return context
 
     def __get_group(self):
         return self.__group
 
     def __set_group(self, group):
         fields = {}
+        fields_views = {}
         if self.group is not None:
             self.group.signal_unconnect(self)
-            for name, field in self.group.fields.iteritems():
+            for name, field in self.group.fields.items():
                 fields[name] = field.attrs
+                fields_views[name] = field.views
         self.tree_states_done.clear()
         self.__group = group
         self.parent = group.parent
@@ -365,6 +384,8 @@ class Screen(SignalEvent):
             self._record_modified)
         self.__group.signal_connect(self, 'group-changed', self._group_changed)
         self.__group.add_fields(fields)
+        for name, views in fields_views.items():
+            self.__group.fields[name].views.update(views)
         self.__group.exclude_field = self.exclude_field
 
     group = property(__get_group, __set_group)
@@ -457,7 +478,7 @@ class Screen(SignalEvent):
                 'model': self.model_name,
                 'id': self.current_record.id if self.current_record else None,
                 'ids': [r.id for r in self.selected_records],
-                }, context=self.context.copy(), warning=False)
+                }, context=self.group._context.copy(), warning=False)
         else:
             if not self.modified():
                 self.switch_view(view_type='form')
@@ -467,7 +488,9 @@ class Screen(SignalEvent):
     def number_of_views(self):
         return len(self.views) + len(self.view_to_load)
 
-    def switch_view(self, view_type=None):
+    def switch_view(self, view_type=None, view_id=None):
+        if view_id is not None:
+            view_id = int(view_id)
         if self.current_view:
             self.current_view.set_value()
             if (self.current_record and
@@ -480,18 +503,32 @@ class Screen(SignalEvent):
                 self.set_cursor()
                 self.current_view.display()
                 return
-        if not view_type or self.current_view.view_type != view_type:
-            for i in xrange(self.number_of_views):
-                if len(self.view_to_load):
-                    self.load_view_to_load()
-                    self.__current_view = len(self.views) - 1
-                else:
-                    self.__current_view = ((self.__current_view + 1)
-                            % len(self.views))
-                if not view_type:
-                    break
-                elif self.current_view.view_type == view_type:
-                    break
+
+        def found():
+            if not self.current_view:
+                return False
+            elif not view_type and view_id is None:
+                return False
+            elif view_id is not None:
+                return self.current_view.view_id == view_id
+            else:
+                return self.current_view.view_type == view_type
+        while not found():
+            if len(self.view_to_load):
+                self.load_view_to_load()
+                self.__current_view = len(self.views) - 1
+            elif (view_id is not None
+                    and view_id not in {v.view_id for v in self.views}):
+                self.add_view_id(view_id, view_type)
+                self.__current_view = len(self.views) - 1
+                break
+            else:
+                self.__current_view = ((self.__current_view + 1)
+                        % len(self.views))
+            if not view_type and view_id is None:
+                break
+            if view_type and not view_id and not len(self.view_to_load):
+                break
         self.screen_container.set(self.current_view.widget)
         self.display()
         # Postpone set of the cursor to ensure widgets are allocated
@@ -542,6 +579,8 @@ class Screen(SignalEvent):
                 fields[field]['loading'] = \
                     self.group.fields[field].attrs['loading']
         self.group.add_fields(fields)
+        for field in fields:
+            self.group.fields[field].views.add(view_id)
         view = View.parse(self, xml_dom, view.get('field_childs'))
         view.view_id = view_id
         self.views.append(view)
@@ -581,11 +620,14 @@ class Screen(SignalEvent):
         if func_name:
             self.group.on_write.add(func_name)
 
-    def cancel_current(self):
+    def cancel_current(self, initial_value=None):
         if self.current_record:
             self.current_record.cancel()
             if self.current_record.id < 0:
-                self.remove(records=[self.current_record])
+                if initial_value is not None:
+                    self.current_record.reset(initial_value)
+                else:
+                    self.remove(records=[self.current_record])
 
     def save_current(self):
         if not self.current_record:
@@ -740,7 +782,6 @@ class Screen(SignalEvent):
                 and not view.attributes.get('tree_state', False)):
             # Mark as done to not set later when the view_type change
             self.tree_states_done.add(id(view))
-            return
         parent = self.parent.id if self.parent else None
         if parent is not None and parent < 0:
             return
@@ -790,13 +831,11 @@ class Screen(SignalEvent):
         self.tree_states_done.add(id(view))
 
     def save_tree_state(self, store=True):
-        if not CONFIG['client.save_tree_state']:
-            return
         parent = self.parent.id if self.parent else None
         timestamp = self.parent._timestamp if self.parent else None
         for view in self.views:
             if view.view_type == 'form':
-                for widgets in view.widgets.itervalues():
+                for widgets in view.widgets.values():
                     for widget in widgets:
                         if hasattr(widget, 'screen'):
                             widget.screen.save_tree_state(store)
@@ -812,7 +851,9 @@ class Screen(SignalEvent):
                 selected_paths = view.get_selected_paths()
                 self.tree_states[parent][view.children_field] = (
                     timestamp, paths, selected_paths)
-                if store and view.attributes.get('tree_state', False):
+                if (store
+                        and int(view.attributes.get('tree_state', False))
+                        and CONFIG['client.save_tree_state']):
                     json_domain = self.get_tree_domain(parent)
                     json_paths = json.dumps(paths, separators=(',', ':'))
                     json_selected_path = json.dumps(
@@ -863,7 +904,9 @@ class Screen(SignalEvent):
             self.search_active(self.current_view.view_type
                 in ('tree', 'graph', 'calendar'))
             for view in self.views:
-                view.display()
+                if (view == self.current_view
+                        or view.widget.get_parent()):
+                    view.display()
             self.current_view.widget.set_sensitive(
                 bool(self.group
                     or (self.current_view.view_type != 'form')
@@ -1011,7 +1054,7 @@ class Screen(SignalEvent):
             record = self.current_record
         domain_string = _('"%s" is not valid according to its domain')
         domain_parser = DomainParser(
-            {n: f.attrs for n, f in record.group.fields.iteritems()})
+            {n: f.attrs for n, f in record.group.fields.items()})
         fields = []
         for field, invalid in sorted(record.invalid_fields.items()):
             string = record.group.fields[field].attrs['string']
@@ -1099,7 +1142,7 @@ class Screen(SignalEvent):
 
     def _button_class(self, button):
         ids = [r.id for r in self.selected_records]
-        context = self.context.copy()
+        context = self.context
         context['_timestamp'] = {}
         for record in self.selected_records:
             context['_timestamp'].update(record.get_timestamp())
@@ -1109,7 +1152,7 @@ class Screen(SignalEvent):
         except RPCException:
             action = None
         self.reload(ids, written=True)
-        if isinstance(action, basestring):
+        if isinstance(action, str):
             self.client_action(action)
         elif action:
             Action.execute(action, {
@@ -1139,17 +1182,16 @@ class Screen(SignalEvent):
             self.display_prev()
         elif action == 'close':
             from tryton.gui import Main
-            Main.get_main().sig_win_close()
+            Main().sig_win_close()
         elif action.startswith('switch'):
-            _, view_type = action.split(None, 1)
-            self.switch_view(view_type=view_type)
+            self.switch_view(*action.split(None, 2)[1:])
         elif action == 'reload':
             if (self.current_view.view_type in ['tree', 'graph', 'calendar']
                     and not self.parent):
                 self.search_filter()
         elif action == 'reload menu':
             from tryton.gui import Main
-            RPCContextReload(Main.get_main().sig_win_menu)
+            RPCContextReload(Main().sig_win_menu)
         elif action == 'reload context':
             RPCContextReload()
 
@@ -1158,16 +1200,17 @@ class Screen(SignalEvent):
         if self.domain:
             query_string.append(('domain', json.dumps(
                         self.domain, cls=JSONEncoder, separators=(',', ':'))))
-        if self.context:
+        context = self.group._context  # Avoid rpc context
+        if context:
             query_string.append(('context', json.dumps(
-                        self.context, cls=JSONEncoder, separators=(',', ':'))))
+                        context, cls=JSONEncoder, separators=(',', ':'))))
         if self.context_screen:
             query_string.append(
                 ('context_model', self.context_screen.model_name))
         if name:
             query_string.append(
                 ('name', json.dumps(name, separators=(',', ':'))))
-        path = [rpc._DATABASE, 'model', self.model_name]
+        path = [CONFIG['login.db'], 'model', self.model_name]
         view_ids = [v.view_id for v in self.views] + self.view_ids
         if self.current_view.view_type != 'form':
             search_string = self.screen_container.get_text()
@@ -1183,7 +1226,7 @@ class Screen(SignalEvent):
         if view_ids:
             query_string.append(('views', json.dumps(
                         view_ids, separators=(',', ':'))))
-        query_string = urllib.urlencode(query_string)
-        return urlparse.urlunparse(('tryton',
-                '%s:%s' % (rpc._HOST, rpc._PORT),
+        query_string = urllib.parse.urlencode(query_string)
+        return urllib.parse.urlunparse(('tryton',
+                CONFIG['login.host'],
                 '/'.join(path), query_string, '', ''))
