@@ -10,17 +10,18 @@ import gettext
 from decimal import Decimal
 
 from .widget import Widget
-from tryton.config import CONFIG
 from tryton.gui.window.win_search import WinSearch
-from tryton.common import RPCExecute, RPCException, Tooltips, \
-    timezoned_date, untimezoned_date
-from tryton.common.placeholder_entry import PlaceholderEntry
+from tryton.common import Tooltips, timezoned_date, untimezoned_date, \
+        IconFactory
 from tryton.common.selection import selection_shortcuts
 from tryton.common.completion import get_completion, update_completion
 from tryton.common.datetime_ import Date, DateTime
 from tryton.common.domain_parser import quote
 from tryton.common.entry_position import reset_position
-from tryton.common.widget_style import set_widget_style
+from tryton.common.underline import set_underline
+from tryton.common.domain_inversion import eval_domain
+from tryton.common.widget_style import widget_class
+from tryton.pyson import PYSONDecoder
 
 _ = gettext.gettext
 
@@ -31,7 +32,7 @@ class DictEntry(object):
 
     def __init__(self, name, parent_widget):
         self.name = name
-        self.definition = parent_widget.keys[name]
+        self.definition = parent_widget.field.keys[name]
         self.parent_widget = parent_widget
         self.widget = self.create_widget()
 
@@ -54,7 +55,6 @@ class DictEntry(object):
 
     def set_readonly(self, readonly):
         self.widget.set_editable(not readonly)
-        set_widget_style(self.widget, not readonly)
 
 
 class DictBooleanEntry(DictEntry):
@@ -127,7 +127,7 @@ class DictSelectionEntry(DictEntry):
         text = child.get_text()
         value = None
         if text:
-            for txt, val in self._selection.items():
+            for txt, val in list(self._selection.items()):
                 if not val:
                     continue
                 if txt[:len(text)].lower() == text.lower():
@@ -319,13 +319,14 @@ class DictWidget(Widget):
     def __init__(self, view, attrs):
         super(DictWidget, self).__init__(view, attrs)
         self.schema_model = attrs['schema_model']
-        self.keys = {}
         self.fields = {}
         self.buttons = {}
         self.rows = {}
 
         self.widget = gtk.Frame()
-        self.widget.set_label(attrs.get('string', ''))
+        label = gtk.Label(set_underline(attrs.get('string', '')))
+        label.set_use_underline(True)
+        self.widget.set_label_widget(label)
         self.widget.set_shadow_type(gtk.SHADOW_OUT)
 
         vbox = gtk.VBox()
@@ -339,11 +340,12 @@ class DictWidget(Widget):
 
         hbox = gtk.HBox()
         hbox.set_border_width(2)
-        self.wid_text = PlaceholderEntry()
+        self.wid_text = gtk.Entry()
         self.wid_text.set_placeholder_text(_('Search'))
         self.wid_text.props.width_chars = 13
         self.wid_text.connect('activate', self._sig_activate)
         hbox.pack_start(self.wid_text, expand=True, fill=True)
+        label.set_mnemonic_widget(self.wid_text)
 
         if int(self.attrs.get('completion', 1)):
             self.wid_completion = get_completion(search=False, create=False)
@@ -356,10 +358,8 @@ class DictWidget(Widget):
 
         self.but_add = gtk.Button()
         self.but_add.connect('clicked', self._sig_add)
-        img_add = gtk.Image()
-        img_add.set_from_stock('tryton-list-add', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_add.set_alignment(0.5, 0.5)
-        self.but_add.add(img_add)
+        self.but_add.add(IconFactory.get_image(
+                'tryton-add', gtk.ICON_SIZE_SMALL_TOOLBAR))
         self.but_add.set_relief(gtk.RELIEF_NONE)
         hbox.pack_start(self.but_add, expand=False, fill=False)
         hbox.set_focus_chain([self.wid_text])
@@ -372,13 +372,14 @@ class DictWidget(Widget):
         self._readonly = False
         self._record_id = None
 
+    @property
+    def _invalid_widget(self):
+        return self.wid_text
+
     def _new_remove_btn(self):
         but_remove = gtk.Button()
-        img_remove = gtk.Image()
-        img_remove.set_from_stock('tryton-list-remove',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_remove.set_alignment(0.5, 0.5)
-        but_remove.add(img_remove)
+        but_remove.add(IconFactory.get_image(
+                'tryton-remove', gtk.ICON_SIZE_SMALL_TOOLBAR))
         but_remove.set_relief(gtk.RELIEF_NONE)
         return but_remove
 
@@ -388,7 +389,7 @@ class DictWidget(Widget):
 
     def _sig_add(self, *args):
         context = self.field.get_context(self.record)
-        value = self.wid_text.get_text().decode('utf-8')
+        value = self.wid_text.get_text()
         domain = self.field.domain_get(self.record)
 
         def callback(result):
@@ -402,23 +403,16 @@ class DictWidget(Widget):
         win.show()
 
     def add_new_keys(self, ids):
-        context = self.field.get_context(self.record)
+        new_keys = self.field.add_new_keys(ids, self.record)
         self.send_modified()
-        try:
-            new_fields = RPCExecute('model', self.schema_model,
-                'get_keys', ids, context=context)
-        except RPCException:
-            new_fields = []
         focus = False
-        for new_field in new_fields:
-            if new_field['name'] not in self.fields:
-                self.keys[new_field['name']] = new_field
-                self.add_line(new_field['name'])
+        for key_name in new_keys:
+            if key_name not in self.fields:
+                self.add_line(key_name)
                 if not focus:
                     # Use idle add because it can be called from the callback
                     # of WinSearch while the popup is still there
-                    gobject.idle_add(
-                        self.fields[new_field['name']].widget.grab_focus)
+                    gobject.idle_add(self.fields[key_name].widget.grab_focus)
                     focus = True
 
     def _sig_remove(self, button, key, modified=True):
@@ -437,20 +431,20 @@ class DictWidget(Widget):
 
     def get_value(self):
         return dict((key, widget.get_value())
-            for key, widget in self.fields.items())
+            for key, widget in list(self.fields.items()))
 
     @property
     def modified(self):
         if self.record and self.field:
             value = self.field.get_client(self.record)
             return any(widget.modified(value)
-                for widget in self.fields.itervalues())
+                for widget in self.fields.values())
         return False
 
     def _readonly_set(self, readonly):
         self._readonly = readonly
         self._set_button_sensitive()
-        for widget in self.fields.values():
+        for widget in list(self.fields.values()):
             widget.set_readonly(readonly)
         self.wid_text.set_sensitive(not readonly)
         self.wid_text.set_editable(not readonly)
@@ -459,13 +453,14 @@ class DictWidget(Widget):
         self.but_add.set_sensitive(bool(
                 not self._readonly
                 and self.attrs.get('create', True)))
-        for button in self.buttons.itervalues():
+        for button in self.buttons.values():
             button.set_sensitive(bool(
                     not self._readonly
                     and self.attrs.get('delete', True)))
 
     def add_line(self, key):
-        self.fields[key] = DICT_ENTRIES[self.keys[key]['type_']](key, self)
+        key_schema = self.field.keys[key]
+        self.fields[key] = DICT_ENTRIES[key_schema['type_']](key, self)
         field = self.fields[key]
         alignment = gtk.Alignment(
             float(self.attrs.get('xalign', 0.0)),
@@ -478,10 +473,11 @@ class DictWidget(Widget):
         n_rows = self.table.props.n_rows
         self.table.resize(n_rows + 1, 3)
         if gtk.widget_get_default_direction() == gtk.TEXT_DIR_RTL:
-            text = _(':') + self.keys[key]['string']
+            text = _(':') + key_schema['string']
         else:
-            text = self.keys[key]['string'] + _(':')
-        label = gtk.Label(text)
+            text = key_schema['string'] + _(':')
+        label = gtk.Label(set_underline(text))
+        label.set_use_underline(True)
         label.set_alignment(1., .5)
         self.table.attach(label, 0, 1, n_rows - 1, n_rows,
             xoptions=gtk.FILL, yoptions=False, xpadding=2)
@@ -492,7 +488,7 @@ class DictWidget(Widget):
         alignment.show_all()
         remove_but = self._new_remove_btn()
         self.tooltips.set_tip(remove_but, _('Remove "%s"') %
-            self.keys[key]['string'])
+            key_schema['string'])
         self.table.attach(remove_but, 2, 3, n_rows - 1, n_rows,
             xoptions=gtk.FILL, yoptions=False, xpadding=2)
         remove_but.connect('clicked', self._sig_remove, key)
@@ -500,29 +496,6 @@ class DictWidget(Widget):
         self.rows[key] = [label, alignment, remove_but]
         self.buttons[key] = remove_but
 
-    def add_keys(self, keys):
-        context = self.field.get_context(self.record)
-        domain = self.field.domain_get(self.record)
-        batchlen = min(10, CONFIG['client.limit'])
-        for i in xrange(0, len(keys), batchlen):
-            sub_keys = keys[i:i + batchlen]
-            try:
-                key_ids = RPCExecute('model', self.schema_model, 'search',
-                    [('name', 'in', sub_keys), domain], 0,
-                    CONFIG['client.limit'], None, context=context)
-            except RPCException:
-                key_ids = []
-            if not key_ids:
-                continue
-            try:
-                values = RPCExecute('model', self.schema_model,
-                    'get_keys', key_ids, context=context)
-            except RPCException:
-                values = []
-            if not values:
-                continue
-            self.keys.update({k['name']: k for k in values})
-
     def display(self, record, field):
         super(DictWidget, self).display(record, field)
 
@@ -531,22 +504,27 @@ class DictWidget(Widget):
 
         record_id = record.id if record else None
         if record_id != self._record_id:
-            for key in self.fields.keys():
+            for key in list(self.fields.keys()):
                 self._sig_remove(None, key, modified=False)
             self._record_id = record_id
 
         value = field.get_client(record) if field else {}
-        new_key_names = set(value.iterkeys()) - set(self.keys)
+        new_key_names = set(value.keys()) - set(field.keys)
         if new_key_names:
-            self.add_keys(list(new_key_names))
-        for key, val in sorted(value.iteritems()):
-            if key not in self.keys:
+            field.add_keys(list(new_key_names), self.record)
+        decoder = PYSONDecoder()
+        for key, val in sorted(value.items()):
+            if key not in field.keys:
                 continue
             if key not in self.fields:
                 self.add_line(key)
             widget = self.fields[key]
             widget.set_value(val)
             widget.set_readonly(self._readonly)
+            key_domain = decoder.decode(
+                self.field.keys[key].get('domain') or '[]')
+            widget_class(
+                widget.widget, 'invalid', not eval_domain(key_domain, value))
         for key in set(self.fields.keys()) - set(value.keys()):
             self._sig_remove(None, key, modified=False)
 
