@@ -98,6 +98,9 @@ class EditableTreeView(TreeView):
             method = model.append
         new_record = model.group.new()
         res = method(new_record)
+        sequence = self.view.attributes.get('sequence')
+        if sequence:
+            model.group.set_sequence(field=sequence)
         return res
 
     def set_cursor(self, path, focus_column=None, start_editing=False):
@@ -139,7 +142,7 @@ class EditableTreeView(TreeView):
         if event.keyval == gtk.keysyms.Right:
             if isinstance(entry, gtk.Entry):
                 if entry.get_position() >= \
-                        len(entry.get_text().decode('utf-8')) \
+                        len(entry.get_text()) \
                         and not entry.get_selection_bounds():
                     leaving = True
             else:
@@ -177,7 +180,7 @@ class EditableTreeView(TreeView):
                         gobject.idle_add(self.set_cursor, path,
                             self.prev_column(path, column), True)
                     elif keyval in self.leaving_record_events:
-                        fields = self.view.widgets.keys()
+                        fields = list(self.view.widgets.keys())
                         if not record.validate(fields):
                             invalid_fields = record.invalid_fields
                             col = None
